open Eliom
open Eliomduce.Xhtml
open Lwt
open Xhtml1_strict

type table_colour = Red | Yellow | Green | Cyan | Blue | Purple;;

let create_table (colours: table_colour list) (headers: string list) (columns: flows list list): table =
let light = ref false in
let head_colour (c: table_colour): string =
begin
	"background-color: " ^
	(match c with
		Red -> "ff1515"
	| Yellow -> "ffff2a"
	| Green -> "15ff15"
	| Cyan -> "2affff"
	| Blue -> "1515ff"
	| Purple -> "ff2aff")
end
and body_colour (l: bool) (c: table_colour): string =
begin
	"background-color: " ^
	(match c with
		Red -> if l then "ff9494" else "ff6a6a" 
	| Yellow -> if l then "ffffaa" else "ffff7f"
	| Green -> if l then "94ff94" else "6aff6a"
	| Cyan -> if l then "aaffff" else "7fffff"
	| Blue -> if l then "9494ff" else "6a6aff"
	| Purple -> if l then "ffaaff" else "ff7fff")
end in
begin
	(* List.iter (fun c ->
		lines := List.map2 (fun o n ->
			o @ [n] 
		) !lines c
	) (List.tl columns); *)
	{{ <table>
		([<tr>([<th>""] @ {: List.map2 (fun c h -> {{ <th style={: head_colour c :}>{: h :} }}) colours headers :})]
		@
		{:
			List.map (fun l ->
				light := not !light;
				{{ <tr>([<td>""] @ {:
				List.map2 (fun clr fd -> {{ <td style={: body_colour !light clr :}> {: fd :} }}) colours l	
			:}) }}) columns
		:})
	}}
end;;

let scribe_version v r: string =
begin
	match r with
		None -> v
	|	Some x -> (v ^ "-" ^ x)
end;;

let rec lwt_map (f: 'a -> 'b Lwt.t) (l: 'a list): 'b list Lwt.t =
begin
	match l with
		[] -> return []
	| hd::tl ->
			f hd >>=
			(fun hd2 -> lwt_map f tl >>=
			(fun tl2 -> return (hd2::tl2)))
end;;

let rec list_cut (l: 'a list) (start: int) (length: int): 'a list =
begin
	if length = 0 then
		[]
	else if start = 0 then
	begin
		match l with
			[] -> []
		| h::t -> h::(list_cut t 0 (length - 1))
	end
	else
	begin
		match l with
			[] -> []
		| h::t -> list_cut t (start - 1) length
	end
end

