open Lifetime
open Napkin
open Rapids
open Diagnosis
open Eliomduce.Xhtml

let cmp_pkgs vi ri (_, pkg1) (_, pkg2) =
let (v1, r1) = pkg1.pk_version
and (v2, r2) = pkg2.pk_version in
begin
	let x = Version_index.compare_versions vi v1 v2 in
		if x = 0 then Release_index.compare_versions ri r1 r2
		else x
end;;

let availability pkg (ar: archive): lifetime =
let life = ref Lifetime.empty in
begin
	Rapids.iterate_over_archive (fun day set ->
		if Package_set.mem pkg set then
			life := Lifetime.add_day !life day
	) ar;
	!life
end;;

let range_of_list db archive_index (a_list: archive_id list): day * day =
let j0 = ref max_int
and j1 = ref 0 in
begin
	List.iter (fun a ->
		let a_data = Archive_index.data archive_index a in
		let (x0, x1) = archive_range db a_data in
			if x0 < !j0 then j0 := x0;
			if x1 > !j1 then j1 := x1 
	) a_list;
	(!j0, !j1)
end;;

type interval = int * int;;
type health = (int, (Package_set.elt, (Rapids.package_id, (Rapids.unit_id, Rapids.version_number * Rapids.release_number) Napkin.versioned) Diagnosis.reason list) Hashtbl.t) Hashtbl.t

let examine_health db bundle (health_tbl: health): unit =
let a_index = get_archive_index db in
let contents = List.map (fun n ->
	let ar = Archive_index.data a_index (Archive_index.search a_index n) in
	Rapids.get_archive_contents ar (Lifetime.day_of_ymd (2007, 06, 30))) bundle in
let available = List.fold_left (fun pkgs ar -> Package_set.union pkgs ar) Package_set.empty contents in
begin
	List.iter (fun a ->
		let failures = Installability.check_archive db ~indicator:Progress.dummy ~archive:a ~available () in
		let ht = Hashtbl.create (List.length failures) in
		List.iter (fun (p_id, reasons) ->
			Hashtbl.add ht p_id reasons
		) failures;
		Hashtbl.add health_tbl (Lifetime.day_of_ymd (2007, 06, 30)) ht
	) contents
end;;
