(* Base822 *)

(** This module transforms a stream of characters (coming from a Debian package description file)
 *  into a stream of row lists. *)

type row = string * string * string list;;
  (** Field name, first row of the field value, remaining rows of the field value. *)

(*** splitter *)
let splitter line s =
  let b = Buffer.create 16 in
  (* state0: initial state *)
  let rec state0' = parser
    | [< ''\n'; s >] ->
        incr line;
        state0' s
    | [< 'c; s >] -> Buffer.add_char b c; state1 [] s
    | [< >] -> [< >]
  (* state0: initial state *)
  and state0 (l : row list) : char Stream.t -> row list Stream.t = parser
    | [< ''\n'; s >] ->
        incr line;
        [< 'l; state0' s >] (* Empty line separating packages *)
    | [< 'c; s >] -> Buffer.add_char b c; state1 l s
    | [< >] -> [< >]
  (* state1: we are reading the field name *)
  and state1 (l : row list) : char Stream.t -> row list Stream.t = parser
    | [< '':'; s >] ->
      let field = Buffer.contents b in
      Buffer.clear b;
      state2 l field s
    | [< ' (('a'..'z'|'A'..'Z'|'-'|'0'..'9') as c); s >] ->
        Buffer.add_char b c;
        state1 l s
  (* state2: we are skipping the spaces between the field name and the value *)
  and state2 (l : row list) (field : string) : char Stream.t -> row list Stream.t = parser
    | [< ' (' '|'\t'); s >] -> state2 l field s
    | [< ''\n'; s >] ->
        incr line;
        state3 l field s
    | [< ' c; s >] -> Buffer.add_char b c; state3 l field s
  (* state3: we are reading the field value in the first row *)
  and state3 (l : row list) (field : string) : char Stream.t -> row list Stream.t = parser
    | [< ''\n'; s >] ->
        incr line;
        let first = Buffer.contents b in
        Buffer.clear b;
        state4 l field first [] s
    | [< 'c; s >] ->
        Buffer.add_char b c;
        state3 l field s
  (* state4: we are looking if there are any extended field values *)
  and state4 (l : row list) (field : string) (first : string) (rest : string list) : char Stream.t -> row list Stream.t = parser
    | [< ' (' '|'\t'); s >] -> Buffer.clear b; state5 l field first rest s
    | [< ''\n'; s >] ->
        incr line;
        Buffer.clear b;
        [< '((field,first,List.rev rest)::l); state0 [] s >]
    | [< 'c; s >] ->
        Buffer.clear b;
        Buffer.add_char b c;
        state0 ((field,first,List.rev rest)::l) s
    | [< >] ->
        Buffer.clear b;
        [< '((field,first,List.rev rest)::l) >]
  and state5 (l : row list) (field : string) (first : string) (rest : string list) : char Stream.t -> row list Stream.t = parser
    | [< ''\n'; s >] ->
        incr line;
        let line = Buffer.contents b in
        state4 l field first
          ((if line = "." then "" else line)::rest) s
    | [< ' c; s >] -> Buffer.add_char b c; state5 l field first rest s
  in
  state0' s
;;
(* ***)

let row_stream_of_char_stream ?(line=ref 0) s = splitter line s;;
