(* Link *)
(* The EDOS project                                                 *)
(* Copyright (C)2000-2006 Berke Durak                               *)
(* Released under the GNU Lesser General Public License version 2.1 *)

type url = string;;

type data =
| Int of int
| Date of int * int * int
| String of string
| Bool of bool
;;

type target =
| Internal of string * (string * data) list 
| External of url
;;

let login    = "login";;
let logout   = Internal("logout",[]);;
let index    = Internal("index",[]);;
let register = Internal("register",[]);;
let who      = Internal("who",[]);;
let query    = Internal("query",[]);;
let console  = Internal("console",[]);;
let credits  = Internal("credits",[]);;
let dql_help = External("http://gallium.inria.fr/~durak/dql.html");;

let string_of_data = function
| Int x -> string_of_int x
| Date(y,m,d) -> Util.string_of_iso_8601 (y,m,d)
| String w -> w
| Bool b -> string_of_bool b
;;

let link ?implicit target =
  match target with
  | External u -> u
  | Internal(u, dl) ->
    let buffer = Buffer.create 256 in
    Buffer.add_string buffer u;
    Buffer.add_char buffer '?';
    let dl = 
      match implicit with
      | None -> dl
      | Some l -> l @ dl
    in
    let dl = List.map (fun (name, data) -> (name, [string_of_data data])) dl in
    Form.encode_form_from_list ~buffer dl
;;

let slink ?sinfo target =
  link ~implicit:
    begin match sinfo with
    | Some(_,s) -> ["sid", String s.Sessions.si_sid]
    | None -> []
    end
    target
;;

let lifetime ~bundle ~architecture ~package_id =
  Internal("lifetime",
            ["bundle",       String bundle;
             "architecture", String architecture;
             "package_id",   Int package_id])
;;
